@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020 NXP
+# Copyright 2020-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,7 +11,7 @@ import logging
 import struct
 import sys
 
-from typing import List
+from typing import List, Dict
 import click
 
 from spsdk import __version__ as spsdk_version
@@ -20,13 +20,38 @@ from spsdk.dat import (DebugAuthenticationChallenge, DebugCredential,
                        DebugAuthenticateResponse, dm_commands)
 from spsdk.dat.debug_mailbox import DebugMailbox
 from spsdk.debuggers.utils import DebugProbeUtils
-from spsdk.debuggers.debug_probe import DebugProbeError
+from spsdk.exceptions import SPSDKError
 
 logger = logging.getLogger("DebugMBox")
 
 LOG_LEVEL_NAMES = [name.lower() for name in logging._nameToLevel]
 PROTOCOL_VERSIONS = ['1.0', '1.1', '2.0', '2.1', '2.2']
 
+def _open_debugmbox(pass_obj: Dict) -> DebugMailbox:
+    """Method opens DebugMailbox object based on input arguments.
+
+    :param pass_obj: Input dictionary with arguments.
+    :return: Active DebugMailbox object.
+    :raise SPSDKError: Raised with any kind of problems with debug probe.
+    """
+    interface = pass_obj['interface']
+    serial_no = pass_obj['serial_no']
+    debug_probe_params = pass_obj['debug_probe_params']
+    timing = pass_obj['timing']
+    reset = pass_obj['reset']
+
+    debug_probes = DebugProbeUtils.get_connected_probes(interface=interface,
+                                                        hardware_id=serial_no,
+                                                        user_params=debug_probe_params)
+    selected_probe = debug_probes.select_probe()
+    debug_probe = selected_probe.get_probe(debug_probe_params)
+    debug_probe.open()
+
+    return DebugMailbox(
+        debug_probe=debug_probe,
+        reset=reset,
+        moredelay=timing
+        )
 
 @click.group()
 @click.option('-i', '--interface')
@@ -52,36 +77,22 @@ def main(ctx: click.Context, interface: str, protocol: str, log_level: str, timi
     logging.basicConfig(level=log_level.upper())
     logger.setLevel(level=log_level.upper())
 
-    if '--help' not in click.get_os_args():
-        # Get the Debug probe object
-        try:
-            probe_user_params = {}
-            for par in debug_probe_option:
-                if par.count("=") != 1:
-                    logger.warning(f"Invalid -o parameter {par}!")
-                else:
-                    par_splitted = par.split("=")
-                    probe_user_params[par_splitted[0]] = par_splitted[1]
+    probe_user_params = {}
+    for par in debug_probe_option:
+        if par.count("=") != 1:
+            raise SPSDKError(f"Invalid -o parameter {par}!")
 
-            debug_probes = DebugProbeUtils.get_connected_probes(interface=interface,
-                                                                hardware_id=serial_no,
-                                                                user_params=probe_user_params)
-            selected_probe = debug_probes.select_probe()
-            debug_probe = DebugProbeUtils.get_probe(interface=selected_probe.interface,
-                                                    hardware_id=selected_probe.hardware_id,
-                                                    user_params=probe_user_params)
-            debug_probe.open()
+        par_splitted = par.split("=")
+        probe_user_params[par_splitted[0]] = par_splitted[1]
 
-            ctx.obj = {
-                'protocol': protocol,
-                'debug_mailbox':
-                    DebugMailbox(
-                        debug_probe=debug_probe, reset=reset, moredelay=timing
-                    ),
-            }
-
-        except DebugProbeError as exc:
-            logger.error(str(exc))
+    ctx.obj = {
+        'protocol': protocol,
+        'interface': interface,
+        'serial_no': serial_no,
+        'debug_probe_params': probe_user_params,
+        'timing': timing,
+        'reset': reset,
+        }
 
     return 0
 
@@ -96,7 +107,7 @@ def auth(pass_obj: dict, beacon: int, certificate: str, key: str, force: bool) -
     """Perform the Debug Authentication."""
     try:
         logger.info("Starting Debug Authentication")
-        mail_box = pass_obj['debug_mailbox']
+        mail_box = _open_debugmbox(pass_obj)
         with open(certificate, 'rb') as f:
             debug_cred_data = f.read()
         debug_cred = DebugCredential.parse(debug_cred_data)
@@ -129,7 +140,7 @@ def auth(pass_obj: dict, beacon: int, certificate: str, key: str, force: bool) -
 def start(pass_obj: dict) -> None:
     """Start DebugMailBox."""
     try:
-        dm_commands.StartDebugMailbox(dm=pass_obj['debug_mailbox']).run()
+        dm_commands.StartDebugMailbox(dm=_open_debugmbox(pass_obj)).run()
         logger.info("Start Debug Mailbox successful")
     except:
         logger.error("Start Debug Mailbox failed!")
@@ -140,7 +151,7 @@ def start(pass_obj: dict) -> None:
 def exit(pass_obj: dict) -> None:
     """Exit DebugMailBox."""
     try:
-        dm_commands.ExitDebugMailbox(dm=pass_obj['debug_mailbox']).run()
+        dm_commands.ExitDebugMailbox(dm=_open_debugmbox(pass_obj)).run()
         logger.info("Exit Debug Mailbox successful")
     except:
         logger.error("Exit Debug Mailbox failed!")
@@ -151,7 +162,7 @@ def exit(pass_obj: dict) -> None:
 def erase(pass_obj: dict) -> None:
     """Erase Flash."""
     try:
-        dm_commands.EraseFlash(dm=pass_obj['debug_mailbox']).run()
+        dm_commands.EraseFlash(dm=_open_debugmbox(pass_obj)).run()
         logger.info("Mass flash erase successful")
     except:
         logger.error("Mass flash erase failed!")
@@ -162,7 +173,7 @@ def erase(pass_obj: dict) -> None:
 def famode(pass_obj: dict) -> None:
     """Set Fault Analysis Mode."""
     try:
-        dm_commands.SetFaultAnalysisMode(dm=pass_obj['debug_mailbox']).run()
+        dm_commands.SetFaultAnalysisMode(dm=_open_debugmbox(pass_obj)).run()
         logger.info("Set fault analysis mode successful")
     except:
         logger.error("Set fault analysis mode failed!")
@@ -174,14 +185,14 @@ def famode(pass_obj: dict) -> None:
 def ispmode(pass_obj: dict, mode: int) -> None:
     """Enter ISP Mode."""
     try:
-        dm_commands.EnterISPMode(dm=pass_obj['debug_mailbox']).run([mode])
+        dm_commands.EnterISPMode(dm=_open_debugmbox(pass_obj)).run([mode])
         logger.info("ISP mode entered successfully!")
     except:
         logger.error("Entering into ISP mode failed!")
 
 
 @catch_spsdk_error
-def safe_main() -> int:
+def safe_main() -> None:
     """Call the main function."""
     sys.exit(main())  # pragma: no cover  # pylint: disable=no-value-for-parameter
 
