@@ -1,30 +1,36 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020 NXP
+# Copyright 2020-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for DebugMailbox PyLink Debug probes support."""
 
 import logging
 from time import sleep
-from typing import List, Dict
+from typing import Dict
 
 import pylink
 import pylink.protocols.swd as swd
 from pylink.errors import JLinkException
 
 from .debug_probe import (DebugProbe,
-                          ProbeDescription,
                           DebugProbeTransferError,
                           DebugProbeNotOpenError,
-                          DebugProbeError)
+                          DebugProbeError,
+                          DebugProbeMemoryInterfaceNotEnabled)
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.CRITICAL)
 JLINK_LOGGER = logger.getChild("PyLink")
 
+def set_logger(level: int) -> None:
+    """Sets the log level for this module.
 
+    param level: Requested level.
+    """
+    logger.setLevel(level)
+
+set_logger(logging.ERROR)
 
 class DebugProbePyLink(DebugProbe):
     """Class to define PyLink package interface for NXP SPSDK."""
@@ -49,7 +55,11 @@ class DebugProbePyLink(DebugProbe):
         """
         super().__init__(hardware_id, user_params)
 
+        set_logger(logging.root.level)
+
+        self.enabled_memory_interface = False
         self.pylink = None
+        self.last_accessed_ap = -1
 
         # Use coresight_read/write API - True (default)
         # or the original swd interface- False (this did not work properly)
@@ -58,17 +68,18 @@ class DebugProbePyLink(DebugProbe):
         logger.debug(f"The SPSDK PyLink Interface has been initialized")
 
     @classmethod
-    def get_connected_probes(cls, hardware_id: str = None, user_params: Dict = None) -> List[ProbeDescription]:
+    def get_connected_probes(cls, hardware_id: str = None, user_params: Dict = None) -> list:
         """Get all connected probes over PyLink.
 
         This functions returns the list of all connected probes in system by PyLink package.
+
         :param hardware_id: None to list all probes, otherwise the the only probe with matching
             hardware id is listed.
         :param user_params: The user params dictionary
         :return: probe_description
         """
         #TODO fix problems with cyclic import
-        from .utils import DebugProbes
+        from .utils import DebugProbes, ProbeDescription
 
         jlink = DebugProbePyLink.get_jlink_lib()
 
@@ -88,8 +99,7 @@ class DebugProbePyLink(DebugProbe):
         The PyLink opening function for SPSDK library to support various DEBUG PROBES.
         The function is used to initialize the connection to target and enable using debug probe
         for DAT purposes.
-        :raises ProbeNotFoundError: The probe has not found
-        :raises DebugMailBoxAPNotFoundError: The debug mailbox access port NOT found
+
         :raises DebugProbeError: The PyLink cannot establish communication with target
         """
         try:
@@ -120,6 +130,24 @@ class DebugProbePyLink(DebugProbe):
         except JLinkException as exc:
             raise DebugProbeError(f"PyLink cannot establish communication with target({str(exc)}).")
 
+    def enable_memory_interface(self) -> None:
+        """Debug probe enabling memory interface.
+
+        General memory interface enabling method (it should be called after open method) for SPSDK library
+        to support various DEBUG PROBES. The function is used to initialize the target memory interface
+        and enable using memory access of target over debug probe.
+
+        :raises DebugProbeNotOpenError: The PyLink probe is NOT opened
+        :raises DebugProbeError: Error with connection to target.
+        """
+        if self.pylink is None:
+            raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
+        try:
+            self.pylink.connect(chip_name="Cortex-M33")
+            self.enabled_memory_interface = True
+        except JLinkException as exc:
+            raise DebugProbeError(f"PyLink cannot establish connection with target({str(exc)}).")
+
     def close(self) -> None:
         """Close PyLink interface.
 
@@ -132,26 +160,78 @@ class DebugProbePyLink(DebugProbe):
         """Read debug mailbox access port register.
 
         This is read debug mailbox register function for SPSDK library to support various DEBUG PROBES.
+
         :param addr: the register address
         :return: The read value of addressed register (4 bytes)
-        :raises NotImplementedError: The dbgmlbx_reg_read is NOT implemented
         """
-        return self._coresight_reg_read(addr=addr)
+        return self.coresight_reg_read(addr=addr | (self.dbgmlbx_ap_ix << self.APSEL_SHIFT))
 
     def dbgmlbx_reg_write(self, addr: int = 0, data: int = 0) -> None:
         """Write debug mailbox access port register.
 
         This is write debug mailbox register function for SPSDK library to support various DEBUG PROBES.
+
         :param addr: the register address
         :param data: the data to be written into register
-        :raises NotImplementedError: The dbgmlbx_reg_write is NOT implemented
         """
-        self._coresight_reg_write(addr=addr, data=data)
+        self.coresight_reg_write(addr=addr | (self.dbgmlbx_ap_ix << self.APSEL_SHIFT), data=data)
 
-    def _coresight_reg_read(self, access_port: bool = True, addr: int = 0) -> int:
+    def mem_reg_read(self, addr: int = 0) -> int:
+        """Read 32-bit register in memory space of MCU.
+
+        This is read 32-bit register in memory space of MCU function for SPSDK library
+        to support various DEBUG PROBES.
+
+        :param addr: the register address
+        :return: The read value of addressed register (4 bytes)
+        :raises DebugProbeNotOpenError: The PyLink probe is NOT opened
+        :raises DebugProbeMemoryInterfaceNotEnabled: The PyLink is using just CoreSight access.
+        """
+        if self.pylink is None:
+            raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
+
+        if not self.enabled_memory_interface:
+            raise DebugProbeMemoryInterfaceNotEnabled("Memory interface is not enabled over J-Link.")
+
+        self.last_accessed_ap = -1
+        reg = [0]
+        try:
+            reg = self.pylink.memory_read32(addr=addr, num_words=1)
+        except JLinkException as exc:
+            logger.error(f"Failed read memory({str(exc)}).")
+        return reg[0]
+
+
+    def mem_reg_write(self, addr: int = 0, data: int = 0) -> None:
+        """Write 32-bit register in memory space of MCU.
+
+        This is write 32-bit register in memory space of MCU function for SPSDK library
+        to support various DEBUG PROBES.
+
+        :param addr: the register address
+        :param data: the data to be written into register
+        :raises DebugProbeNotOpenError: The PyLink probe is NOT opened
+        :raises DebugProbeMemoryInterfaceNotEnabled: The PyLink is using just CoreSight access.
+        """
+        if self.pylink is None:
+            raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
+
+        if not self.enabled_memory_interface:
+            raise DebugProbeMemoryInterfaceNotEnabled("Memory interface is not enabled over J-Link.")
+
+        self.last_accessed_ap = -1
+        try:
+            data_list = list()
+            data_list.append(data)
+            self.pylink.memory_write32(addr=addr, data=data_list)
+        except JLinkException as exc:
+            logger.error(f"Failed write memory({str(exc)}).")
+
+    def coresight_reg_read(self, access_port: bool = True, addr: int = 0) -> int:
         """Read coresight register over PyLink interface.
 
         The PyLink read coresight register function for SPSDK library to support various DEBUG PROBES.
+
         :param access_port: if True, the Access Port (AP) register will be read(defau1lt), otherwise the Debug Port
         :param addr: the register address
         :return: The read value of addressed register (4 bytes)
@@ -163,6 +243,12 @@ class DebugProbePyLink(DebugProbe):
             raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
 
         try:
+            if access_port:
+                req_ap = (self.APSEL & addr) >> self.APSEL_SHIFT
+                if self.last_accessed_ap != req_ap:
+                    self._select_ap(req_ap)
+                    self.last_accessed_ap = req_ap
+
             if not self.use_coresight_rw:
                 request = swd.ReadRequest(addr // 4, ap=access_port)
                 response = request.send(self.pylink)
@@ -177,10 +263,11 @@ class DebugProbePyLink(DebugProbe):
         except JLinkException as exc:
             raise DebugProbeTransferError(f"The Coresight read operation failed({str(exc)}).")
 
-    def _coresight_reg_write(self, access_port: bool = True, addr: int = 0, data: int = 0) -> None:
+    def coresight_reg_write(self, access_port: bool = True, addr: int = 0, data: int = 0) -> None:
         """Write coresight register over PyLink interface.
 
         The PyLink write coresight register function for SPSDK library to support various DEBUG PROBES.
+
         :param access_port: if True, the Access Port (AP) register will be write(default), otherwise the Debug Port
         :param addr: the register address
         :param data: the data to be written into register
@@ -191,6 +278,12 @@ class DebugProbePyLink(DebugProbe):
             raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
 
         try:
+            if access_port:
+                req_ap = (self.APSEL & addr) >> self.APSEL_SHIFT
+                if self.last_accessed_ap != req_ap:
+                    self._select_ap(req_ap)
+                    self.last_accessed_ap = req_ap
+
             if not self.use_coresight_rw:
                 request = swd.WriteRequest(addr // 4, data=data, ap=access_port)
                 response = request.send(self.pylink)
@@ -202,18 +295,31 @@ class DebugProbePyLink(DebugProbe):
         except JLinkException as exc:
             raise DebugProbeTransferError(f"The Coresight write operation failed({str(exc)}).")
 
+    def reset(self) -> None:
+        """Reset a target.
+
+        It resets a target.
+
+        :raises DebugProbeNotOpenError: The PyLink probe is NOT opened
+        """
+        if self.pylink is None:
+            raise DebugProbeNotOpenError("The PyLink debug probe is not opened yet")
+
+        self.pylink.reset()
+
     def _select_ap(self, ap_ix: int, address: int = 0) -> None:
         """Helper function to selct the access port in DP.
 
         :param ap_ix: requested Access port  index.
         :param address: requested address.
         """
-        self._coresight_reg_write(access_port=False, addr=0x08, data=(address | (ap_ix << 24)))
+        self.coresight_reg_write(access_port=False, addr=0x08, data=(address | (ap_ix << 24)))
 
     def _get_dmbox_ap(self) -> int:
         """Search for Debug Mailbox Access Point.
 
         This is helper function to find and return the debug mailbox access port index.
+
         :return: Debug MailBox Access Port Index if found, otherwise -1
         :raises DebugProbeNotOpenError: The Segger JLink probe is NOT opened
         """
@@ -228,7 +334,7 @@ class DebugProbePyLink(DebugProbe):
         for access_port_ix in range(256):
             try:
                 self._select_ap(ap_ix=access_port_ix, address=0x000000F0)
-                ret = self._coresight_reg_read(addr=idr_address)
+                ret = self.coresight_reg_read(addr=idr_address)
 
                 if ret == idr_expected:
                     logger.debug(f"Found debug mailbox ix:{access_port_ix}")
