@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2020 NXP
+# Copyright 2019-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -17,7 +17,7 @@ from crccheck.crc import Crc16
 from serial import Serial, SerialException
 from serial.tools.list_ports import comports
 
-from spsdk.mboot.exceptions import McuBootConnectionError
+from spsdk.mboot.exceptions import McuBootConnectionError, McuBootDataAbortError
 from spsdk.mboot.commands import CmdPacket, CmdResponse, parse_cmd_response
 from spsdk.utils.easy_enum import Enum
 
@@ -60,8 +60,8 @@ def _check_port(port: str, baudrate: int, timeout: int) -> Optional[Interface]:
         interface.open()
         interface.close()
         return interface
-    except (AssertionError, SerialException) as e:
-        logger.error(str(e))
+    except (AssertionError, SerialException, TimeoutError) as e:
+        logger.debug(f"{type(e).__name__}: {e}")
         return None
 
 
@@ -142,7 +142,7 @@ class Uart(Interface):
         :type timeout: int, optional
         """
         super().__init__()
-        self.device = Serial(port=port, timeout=timeout // 1000, baudrate=baudrate)
+        self.device = Serial(port=port, timeout=timeout / 1000, baudrate=baudrate)
         self.close()
         self.protocol_version = None
         self.options = None
@@ -169,10 +169,14 @@ class Uart(Interface):
 
         :return: read data
         :rtype: Union[spsdk.mboot.commands.CmdResponse, bytes]
+        :raises McuBootDataAbortError: Indicates data transmission abort
         """
         _, frame_type = self._read_frame_header()
         length = to_int(self._read(2))
         crc = to_int(self._read(2))
+        if not length:
+            self._send_ack()
+            raise McuBootDataAbortError()
         data = self._read(length)
         self._send_ack()
         calculated_crc = self._calc_frame_crc(data, frame_type)
@@ -200,11 +204,12 @@ class Uart(Interface):
         """Read 'length' amount for bytes from device.
 
         :param length: Number of bytes to read
-        :type length: int
         :return: Data read from the device
-        :rtype: bytes
+        :raises TimeoutError: Time-out
         """
         data = self.device.read(length)
+        if not data:
+            raise TimeoutError()
         logger.debug(f"<{' '.join(f'{b:02x}' for b in data)}>")
         return data
 
@@ -241,12 +246,15 @@ class Uart(Interface):
 
         :param expected_frame_type: Check if the frame_type is exactly as expected
         :return: Tuple of integers representing frame header and frame type
+        :raises McuBootDataAbortError: Target sens Data Abort frame
         :raises AssertionError: Unexpected frame header or frame type (if specified)
         """
         header = to_int(self._read(1))
         assert header == self.FRAME_START_BYTE, \
             f"Received invalid frame header '{header:#X}' expected '{self.FRAME_START_BYTE:#X}'"
         frame_type = to_int(self._read(1))
+        if frame_type == FPType.ABORT:
+            raise McuBootDataAbortError()
         if expected_frame_type:
             assert frame_type == expected_frame_type, \
                 f"received invalid ACK '{frame_type:#X}' expected '{expected_frame_type:#X}'"
@@ -276,9 +284,10 @@ class Uart(Interface):
         return calc_crc(crc_data)
 
     def ping(self) -> None:
-        """Ping the target device, retreive protocol version.
+        """Ping the target device, retrieve protocol version.
 
         :raises AssertionError: If the target device doesn't respond to ping
+        :raises McuBootConnectionError: If the start frame is not received
         """
         ping = struct.pack('<BB', self.FRAME_START_BYTE, FPType.PING)
         self._send_frame(ping, wait_for_ack=False)
