@@ -2,22 +2,15 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2020 NXP
+# Copyright 2019-2021 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
-
 """Module for USB communication with a terget using SDP protocol."""
 
-# This violation is suppressed due to differences in Win/Linux implementation of USB
-# pylint: disable=E1101
-
-import collections
 import logging
-import os
 import platform
 
-from time import time
-from typing import List, Tuple, Union
+from typing import Sequence, Tuple, Union
 
 import hid
 
@@ -29,10 +22,10 @@ from .base import Interface
 
 logger = logging.getLogger('SDP:USB')
 
+# import os
 # os.environ['PYUSB_DEBUG'] = 'debug'
 # os.environ['PYUSB_LOG_FILENAME'] = 'usb.log'
 
-
 HID_REPORT = {
     # name | id | length
     'CMD': (0x01, 1024, False),
@@ -53,16 +46,12 @@ USB_DEVICES = {
     'MX7SD': (0x15A2, 0x0076),
     'MX7ULP': (0x1FC9, 0x0126),
     'VYBRID': (0x15A2, 0x006A),
-
     'MXRT20': (0x1FC9, 0x0130),
     'MXRT50': (0x1FC9, 0x0130),
     'MXRT60': (0x1FC9, 0x0135),
-
     'MX8MQ': (0x1FC9, 0x012B),
-
     'MX8QXP-A0': (0x1FC9, 0x007D),
     'MX8QM-A0': (0x1FC9, 0x0129),
-
     'MX8QXP': (0x1FC9, 0x012F),
     'MX8QM': (0x1FC9, 0x0129),
     'MX815': (0x1FC9, 0x013E),
@@ -70,7 +59,7 @@ USB_DEVICES = {
 }
 
 
-def scan_usb(device_name: str = None) -> List[Interface]:
+def scan_usb(device_name: str = None) -> Sequence[Interface]:
     """Scan connected USB devices. Return a list of all devices found.
 
     :param device_name: see USBDeviceFilter classes constructor for usb_id specification
@@ -85,7 +74,6 @@ def scan_usb(device_name: str = None) -> List[Interface]:
 ########################################################################################################################
 class RawHid(Interface):
     """Base class for OS specific RAW HID Interface classes."""
-
     @property
     def name(self) -> str:
         """Get the name of the device.
@@ -103,14 +91,14 @@ class RawHid(Interface):
 
         :return: True if device is open, False othervise.
         """
-        return self._opened
+        return self.device is not None and self._opened
 
     def __init__(self) -> None:
         """Initialize the USB interface object."""
         self._opened = False
         self.vid = 0
         self.pid = 0
-        self.sn = ""
+        self.serial_number = ""
         self.vendor_name = ""
         self.product_name = ""
         self.interface_number = 0
@@ -150,7 +138,7 @@ class RawHid(Interface):
         """Return information about the USB interface."""
         return f"{self.product_name:s} (0x{self.vid:04X}, 0x{self.pid:04X})"
 
-    def conf(self, config: dict):
+    def conf(self, config: dict) -> None:
         """Set HID report data.
 
         :param config: parameters dictionary
@@ -163,28 +151,38 @@ class RawHid(Interface):
         """Open the interface."""
         logger.debug("Open Interface")
         try:
+            assert self.device
             self.device.open_path(self.path)
             self.device.set_nonblocking(False)
             # self.device.read(1021, 1000)
             self._opened = True
         except OSError:
-            raise SdpConnectionError(f"Unable to open device VIP={self.vid} PID={self.pid} SN='{self.sn}'")
+            raise SdpConnectionError(
+                f"Unable to open device VIP={self.vid} PID={self.pid} SN='{self.serial_number}'"
+            )
 
     def close(self) -> None:
         """Close the interface."""
         logging.debug("Close Interface")
         try:
+            assert self.device
             self.device.close()
             self._opened = False
         except OSError:
-            raise SdpConnectionError(f"Unable to close device VIP={self.vid} PID={self.pid} SN='{self.sn}'")
+            raise SdpConnectionError(
+                f"Unable to close device VIP={self.vid} PID={self.pid} SN='{self.serial_number}'"
+            )
 
     def write(self, packet: Union[CmdPacket, bytes]) -> None:
         """Write data on the OUT endpoint associated to the HID interfaces.
 
         :param packet: Data to send
         :raises ValueError: Raises an error if packet type is incorrect
+        :raises SdpConnectionError: Raises an error if device is openned for writing
         """
+        if not self.is_opened:
+            raise SdpConnectionError(f"Device is openned for writing")
+
         if isinstance(packet, CmdPacket):
             report_id, report_size, hid_ep1 = HID_REPORT['CMD']
             data = packet.to_bytes()
@@ -194,23 +192,30 @@ class RawHid(Interface):
         else:
             raise ValueError("Packet has to be either 'CmdPacket' or 'bytes'")
 
+        assert self.device
         data_index = 0
         while data_index < len(data):
-            raw_data, data_index = self._encode_report(report_id, report_size, data, data_index)
+            raw_data, data_index = self._encode_report(report_id, report_size,
+                                                       data, data_index)
             self.device.write(raw_data)
 
     def read(self, length: int = None) -> CmdResponse:
         """Read data on the IN endpoint associated to the HID interface.
 
         :return: Return CmdResponse object.
+        :raises SdpConnectionError: Raises an error if device is openned for reading
         """
+        if not self.is_opened:
+            raise SdpConnectionError(f"Device is openned for reading")
+
+        assert self.device
         raw_data = self.device.read(1024, self.timeout)
         if raw_data[0] == 0x04 and platform.system() == "Linux":
             raw_data += self.device.read(1024, self.timeout)
         return self._decode_report(bytes(raw_data))
 
     @staticmethod
-    def enumerate(usb_device_filter: USBDeviceFilter) -> List[Interface]:
+    def enumerate(usb_device_filter: USBDeviceFilter) -> Sequence[Interface]:
         """Get list of all connected devices which matches device_id.
 
         :param usb_device_filter: USBDeviceFilter object
